{-# LANGUAGE OverloadedStrings #-}
module Controller.Orphans where

import Data.Aeson.Types
import Text.PrettyPrint.ANSI.Leijen

import Data.Rewriting.Term.Type
import Data.Rewriting.Rule.Type
import Data.Rewriting.Context.Type
import Data.Rewriting.Rules.Rewrite (Reduct (..))

instance (ToJSON f, ToJSON v) => ToJSON (Term f v) where
    toJSON (Var v) = object [ "tag" .= ("Var" :: String)
                            , "name" .= toJSON v
                            ]
    toJSON (Fun name params) =
        object [ "tag" .= ("Fun" :: String)
               , "name" .= toJSON name
               , "params" .= toJSON params
               ]

instance (ToJSON f, ToJSON v, Pretty f, Pretty v) => ToJSON (Rule f v) where
    toJSON rule = object [ "tag" .= ("Rule" :: String)
                         , "lhs" .= (show . prettyTerm $ lhs rule)
                         , "rhs" .= (show . prettyTerm $ rhs rule)
                         ]

instance (ToJSON f, ToJSON v) => ToJSON (Ctxt f v) where
    toJSON Hole =
        object [ "tag" .= ("Hole" :: String)
               , "name" .= ("☐" :: String)
               ]
    toJSON (Ctxt f pref ctxt post) =
        object [ "tag"        .= ("Ctxt" :: String)
               , "name"        .= f
               , "prefix"     .= toJSON pref
               , "subContext" .= toJSON ctxt
               , "postfix"    .= toJSON post
               ]

instance (ToJSON f, ToJSON v, ToJSON v', Pretty f, Pretty v, Pretty v')
  => ToJSON (Reduct f v v') where
    toJSON reduct =
        object [ "result" .= (show . prettyTerm $ result reduct)
               , "pos" .= toJSON (pos reduct)
               , "rule" .= toJSON (rule reduct)
               ]


prettyTerm :: (Pretty f, Pretty v)
           => Term f v -> Doc
prettyTerm (Var v) = pretty v
prettyTerm (Fun f []) = pretty f
prettyTerm (Fun f params@(_:_)) = pretty f <> args
    where
        args = encloseSep lparen rparen comma [prettyTerm ti | ti <- params]

