{-#LANGUAGE OverloadedStrings #-}
module Main where

import Network.Wai.Handler.Warp
import Network.Wai.Middleware.Static
import Network.Wai.Middleware.RequestLogger
import Web.Scotty

import Data.Char (isDigit)
import Data.Default.Class (def)
import System.Environment
import System.Exit

import Controller.Controllers

ops :: Int -> Options
ops p = def { settings = setPort p $ setTimeout 5 $ settings def
            }

-- | Start and run the scotty Server
main :: IO ()
main = do
    args <- getArgs
    p <- case args of
           [parg] | all isDigit parg -> return $ read parg
           [] -> return 3000
           _ -> usage >> exitFailure
    scottyOpts (ops p) $ do
        initMiddleware
        routes

usage :: IO ()
usage = do
    name <- getProgName
    putStrLn $ "usage: " ++ name ++ " [PORT]"
    putStrLn $ ""
    putStrLn $ "PORT\t it the port the application should listen on (default: 3000)"

-- | Routing table
routes :: ScottyM ()
routes = do
    get  "/"                   homeController
    get  "/strategies"         strategiesController
    post "/strategies/suggest" suggestStrategyController
    post "/rewrite"            rewriteController
    post "/annotation"         annotationController
    post "/annotation/suggest" suggestAnnotationController

-- | Middleware layers
initMiddleware :: ScottyM ()
initMiddleware = do
    middleware $ staticPolicy (noDots >-> addBase "static")
    middleware logStdoutDev

