{-# LANGUAGE OverloadedStrings #-}
-- | All the HTML is constructed here
module View.Home (
    -- * Views
    homeView,
    -- * Utils
    templt,
    -- * Reexport
    renderHtml,
    Html
    ) where

import Prelude hiding (head, div)
import Data.List hiding (head)

import Text.Blaze.Html5 as H hiding (map)
import Text.Blaze.Html5.Attributes as A
import Text.Blaze.Html.Renderer.Text

import Controller.Strategies
import Data.String (fromString)

-- | Returns the Homepage
homeView :: Html
homeView = templt $ H.title "Automating Rewrite Strategies"

-- | Homepage template
templt :: Html -> Html
templt hd = H.docTypeHtml $ do
    head $ do
        hd
        H.link ! href "bootstrap.min.css" ! rel "stylesheet" ! type_ "text/css"
        H.link ! href "style.css" ! rel "stylesheet" ! type_ "text/css"
        H.script ! src "https://code.jquery.com/jquery-3.2.1.min.js" $ mempty
    body $ do
        H.div ! class_ "main-content container" $ do
            h1 "Rewriting:"
            inputArea
            resultArea
            instructions
        script ! src "myjs.js" $ mempty


instrCode = H.code ! A.class_ "code-class"

instructions :: Html
instructions = H.div ! A.id "instructions" $ do
    h3 "Format"
    "The main input field accepts an extended format of the one used by "
    H.a ! A.href "http://project-coco.uibk.ac.at/problems/trs.php" $ "COPS."
    " The extension allows for naming rules and adding strategy annotations."
    " It is defined as follows:"
    br
    H.pre $ H.div ! A.id "code-block" $ instrCode $ mconcat . intersperse "\n" $
        [ "trs        ::= [(VAR idlist)] (RULES rulelist) [(ANNOTATION annotation)]"
        , "idlist     ::= _empty_ | id idlist"
        , "rulelist   ::= _empty_ | rule rulelist"
        , "rule       ::= term -> term | id: term -> term"
        , "term       ::= id | id() | id(termlist)"
        , "termlist   ::= term | term, termlist"
        , "annotation ::= _empty_ | (id annotlist) annotation"
        , "annotlist  ::= _empty_ | pos annotlist | id annotlist"
        ]
    mconcat $
        [ "Here "
        , instrCode $ " pos"
        , " is a natural number greater 0, and "
        , instrCode $ "id"
        , " is any sequence of characters not containing whitespace or the characters "
        , instrCode $ "(" , instrCode $ ")" , instrCode $ "," , instrCode $ ":"
        ]

inputArea :: Html
inputArea = do
    H.div ! A.id "inputarea" $ do
        H.form ! name "myform" ! A.id "myform" ! method "POST" $ do
            textarea ! A.id "rewriteSystem"
                     ! name "system"
                     $ defTRS
            br
            H.div ! A.id "inp" $ do
                input ! A.id "term"
                      ! A.type_ "text"
                      ! name "term"
                      ! placeholder "Term"
            H.div ! A.id "buttons" $ do
                input ! type_ "submit" ! value "Rewrite"
                button ! type_ "button" ! onclick "suggestAnnotation()" $ "Suggest Annotation"
                button ! type_ "button" ! onclick "suggestStrategy()" $ "Suggest Strategy"
                select ! name "strat" $ mconcat (
                      (option ! value "annot") "Annotation" :
                      map (\(s,n) -> option ! value n $ s) strats )

defTRS = "(VAR x y)\n(RULES\n  alpha: f(x) -> a\n)\n(ANNOTATION\n (f alpha 1)\n)"

resultArea :: Html
resultArea = do
    H.div ! A.class_ "current" $
        currentArea
    H.div ! A.id "answer" $ do
        code ! A.id "answer-field" $ mempty
        H.button ! A.type_ "button" ! A.id "more-btn" $ "More!"

currentArea :: Html
currentArea = do
    H.div ! A.class_ "current-term" $
        code $ H.span ! A.class_ "redex" $ mempty
    H.div ! A.class_ "controls" $ do
        H.button ! A.type_ "button" ! A.onclick "previouseState()" $ "Previous"
        H.button ! A.type_ "button" ! A.onclick "toggleAnim()" $ "Play/Pause"
        H.button ! A.type_ "button" ! A.onclick "nextState()" $ "Next"
    H.div ! A.class_ "current-info" $ do
        H.ul $ do
            li $ do
                "Result:"
                H.code ! A.class_ "current-info-result" ! A.style "display: none;" $ mempty
            li $ do
                "Rule:"
                H.code ! A.class_ "current-info-rule" ! A.style "display: none;" $ mempty


strats :: [(Html, AttributeValue)]
strats = map (\(na,(i,_)) -> (fromString na,fromString i)) strategies
